import matplotlib
matplotlib.use('Agg')

import argparse
import itertools
import numpy as np
import matplotlib.pyplot as plt


#from sklearn import svm, datasets
#from sklearn.model_selection import train_test_split
from sklearn.metrics import confusion_matrix
from sklearn.metrics import accuracy_score



def plot_confusion_matrix(cm, classes, accuracy, normalize=False, title='Accuracy ', cmap=plt.cm.Blues):
		"""
		This function prints and plots the confusion matrix.
		Normalization can be applied by setting `normalize=True`.
		"""
		
		plt.imshow(cm, interpolation='nearest', cmap=cmap)
		#plt.imshow(cm)
		plt.title(title+str(accuracy))
		plt.colorbar()
		tick_marks = np.arange(len(classes))
		plt.xticks(tick_marks, classes, rotation=0) # rotation=45
		plt.yticks(tick_marks, classes)

		if normalize:
				cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
				print("Normalized confusion matrix")
		else:
				print('Confusion matrix, without normalization')

		print(cm)

		thresh = cm.max() / 2.
		for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
				plt.text(j, i, cm[i, j], horizontalalignment="center", color="white" if cm[i, j] > thresh else "black")

		plt.tight_layout()
		plt.ylabel('True label')
		plt.xlabel('Predicted label')




# parse arguments ------------------------
parser = argparse.ArgumentParser(description='This is an auxiliary tool to analyze results from a classifier ')
parser.add_argument('-pf','--pred_file',help='Prediction file, formatted as y,y\' per line', required=True)
parser.add_argument('-sf','--save_figure',help='File to save the confusion matrix', required=False)

args = parser.parse_args()

m = np.loadtxt(args.pred_file, dtype='int', delimiter=",")

class_names=np.unique(m)

acs=accuracy_score(m[:,1], m[:,0])



## Compute confusion matrix
cnf_matrix = confusion_matrix(m[:,1], m[:,0])
np.set_printoptions(precision=2)
#
## Plot non-normalized confusion matrix
fig=plt.figure()
plot_confusion_matrix(cnf_matrix, classes=class_names.tolist(),accuracy=acs, title='Accuracy: ')
#
## Plot normalized confusion matrix
#plt.figure()
#plot_confusion_matrix(cnf_matrix, classes=class_names, normalize=True,title='Normalized confusion matrix')
#

if args.save_figure:
	fig.savefig(args.save_figure)

print "Accuracy:"
print acs
print "============================"

#plt.show()
