# encoding: UTF-8
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#/nfs/faculty/estrada/proteins

# nohup python keras_model3.py --data_dir /nfs/faculty/estrada/proteins/datasets/splitpdb_new --training_steps 50 --testing_percent 20 --batch_size=100 --save_pred /nfs/faculty/estrada/code/proteins_keras/saverun > outkeras.txt


import cv2
import glob
import numpy as np
import argparse
import os
import random
import math
import pandas as pd

import keras
from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten, Input
from keras.layers import Conv2D, MaxPooling2D
from keras.layers import Concatenate, concatenate, BatchNormalization
from keras.optimizers import SGD
from keras.optimizers import Nadam
from keras.optimizers import RMSprop
from keras.utils import plot_model
from keras.models import Model
import graphviz
import pydot_ng as pydot
import pydot

from sklearn.utils import class_weight



class DataIms(object):
	def __init__(self, X_train, Y_train, X_test, Y_test, n_classes,datadist,names):
		self.Y_train = np.asarray(Y_train, dtype=np.int)
		self.X_train = np.asarray(X_train, dtype=np.float32)
		self.Y_test = np.asarray(Y_test, dtype=np.int)
		self.X_test = np.asarray(X_test, dtype=np.float32)
		self.names_test = names
		self.n_classes=n_classes
		self.datadist=datadist
		self.batch_size = 0

	# returns an iterable in batches

	def yieldbatch(self,iterable, n=1):
		l = len(self.X_train)
		for ndx in range(0, l, n):
			yield iterable[ndx:min(ndx + n, l)]
		
		
	def set_batches(self,batch_size):
		self.batch = iter(list(yieldbatch(range(0,len(self.X_train)),batch_size)))
	 	#USE: batch=set_batches(batch_size)  # next(batch)
	
	
	def preprocess_training(self,randomize,oversample):
		if oversample:
			a = numpy.array(self.datadist)
			nc=len(a)
			diffa=a[1:nc]-a[0:nc-1]
			maxa=max(diffa)
			X_temp=[]
			Y_temp=[]
			for yx in range(nc):
				yinit=a[yx]
				if yx+1 > nc:
					yend=len(self.X_train-1)
				else:
					yend=a[yx+1]
					
				X_temp.append(self.X_train[yinit:yend])
				Y_temp.append(self.Y_train[yinit:yend])
				togo=maxa-diffa[yx]
				while togo > 0:
					if togo > diffa[yx]:
						X_temp.append(self.X_train[yinit:yend])
						Y_temp.append(self.Y_train[yinit:yend])
						togo=togo-diffa[yx]
					else:
						X_temp.append(self.X_train[yinit:yinit+togo])
						Y_temp.append(self.Y_train[yinit:yinit+togo])
						togo=0
		
		if randomize:
			print "randomize"
			ndat=len(self.X_train)
			idx=np.random.permutation(np.arange(ndat))
			# perform a random permutation of training data and class
			self.X_train=self.X_train[idx]
			self.Y_train=self.Y_train[idx]
			self.datadist=[]
			
			
def rand_percent(percent):
	if random.randint(1,101) <= int(percent):
		return True
	return False
		 
def load_images_files_fold(data_dir, flip, mirror,n_fold,fold):
	X_data = []
	Y_data=[]
	X_test = []
	Y_test=[]
	names_test=[]
	datadist=[]
	print n_fold
	print fold
	
	# find out the number of folders, use that to build y
	dirs=next(os.walk(data_dir))[1] # find all the directories in path
	n_classes=len(dirs) # number of classes
	ndir=0;
	cont=0;
	contfold=-1;
	for name in dirs:  # every directory is a class
		print(name)
		files = glob.glob (data_dir+"/"+name+"/*.jpg")
		ytemplate=np.zeros(n_classes)
		ytemplate[ndir]=1 # make the one-hot template for y
		datadist.append(cont) # keep track of where the different classes begin
		
		for myFile in files: # traverse all the files in the directory
			#print(myFile)
			contfold=contfold+1
			image = cv2.imread (myFile) # read the image
			# divide percent for testing and the rest for training
			if contfold % n_fold == fold: 
				X_test.append(image)
				Y_test.append(ytemplate)
				names_test.append(myFile)
			else:
				X_data.append(image)
				Y_data.append(ytemplate)
				cont=cont+1;
				if flip: # flip the image upside down
					fimg=cv2.flip(image,0)
					fimg=cv2.flip(fimg,1)
					X_data.append(fimg)
					Y_data.append(ytemplate)
					cont=cont+1;
				if mirror:  # mirror the image 
					fimg=cv2.flip(image,1)
					X_data.append(fimg)
					Y_data.append(ytemplate)
					cont=cont+1;
		ndir=ndir+1; # index for the next class

	print('X_data shape:', np.array(X_data).shape)
	print('X_test shape:', np.array(X_test).shape)
	print datadist
	return DataIms(X_data,Y_data,X_test,Y_test,n_classes,datadist,names_test)  # dataset = load_images..
	#return X_data


def load_images_files(data_dir, percent, flip, mirror):
	X_data = []
	Y_data=[]
	X_test = []
	Y_test=[]
	names_test=[]
	datadist=[]
	print percent
	
	# find out the number of folders, use that to build y
	dirs=next(os.walk(data_dir))[1] # find all the directories in path
	n_classes=len(dirs) # number of classes
	ndir=0;
	cont=0;
	for name in dirs:  # every directory is a class
		print(name)
		files = glob.glob (data_dir+"/"+name+"/*.jpg")
		ytemplate=np.zeros(n_classes)
		ytemplate[ndir]=1 # make the one-hot template for y
		datadist.append(cont) # keep track of where the different classes begin
		
		for myFile in files: # traverse all the files in the directory
			#print(myFile)
			image = cv2.imread (myFile) # read the image
			# divide percent for testing and the rest for training
			if rand_percent(percent): 
				X_test.append(image)
				Y_test.append(ytemplate)
				names_test.append(myFile)
			else:
				X_data.append(image)
				Y_data.append(ytemplate)
				cont=cont+1;
				if flip: # flip the image upside down
					fimg=cv2.flip(image,0)
					fimg=cv2.flip(fimg,1)
					X_data.append(fimg)
					Y_data.append(ytemplate)
					cont=cont+1;
				if mirror:  # mirror the image 
					fimg=cv2.flip(image,1)
					X_data.append(fimg)
					Y_data.append(ytemplate)
					cont=cont+1;
		ndir=ndir+1; # index for the next class

	print('X_data shape:', np.array(X_data).shape)
	print('X_test shape:', np.array(X_test).shape)
	print datadist
	return DataIms(X_data,Y_data,X_test,Y_test,n_classes,datadist,names_test)  # dataset = load_images..
	#return X_data
        

def parse_args():
	"""Parse command line arguments passed to script invocation."""
	parser = argparse.ArgumentParser(description='Train keras model with protein data.')

	parser.add_argument('--data_dir', help='data directory')
	parser.add_argument('--training_steps', type=int, help='number of training steps')
	parser.add_argument('--testing_percent', default=20, type=int, help='testing percentage')
	parser.add_argument('--batch_size', type=int, help='batch size')
	parser.add_argument('--save_pred', help='file to save prediction')
	parser.add_argument('--n_fold', default=-1, type=int, help='number of folds')
	parser.add_argument('--fold', default=0, type=int, help='specific fold value')
	
	return parser.parse_args()


def main():
	"""Module's main entry point (zopectl.command)."""
	args = parse_args()
	data_dir = args.data_dir
	training_steps = args.training_steps
	testing_percent = args.testing_percent
	save_pred = args.save_pred
	bsize=args.batch_size
	n_fold=args.n_fold
	fold=args.fold
	flip=False
	mirror=False
	randomize=True
	oversample=False

	if not os.path.exists(data_dir):
		raise Exception('Directory does not exist ({0}).'.format(data_dir))
	
	if n_fold>0:
		dataset = load_images_files_fold(os.path.abspath(data_dir), flip, mirror,n_fold,fold)
	else:
		dataset = load_images_files(os.path.abspath(data_dir), testing_percent, flip, mirror)
	
	print dataset.X_train.shape
	ntest, imsizex, imsizey, nchannels = dataset.X_test.shape
	ndata, imsizex, imsizey, nchannels = dataset.X_train.shape

	dataset.batch_size = bsize
	dataset.preprocess_training(randomize,oversample)
	
	imsize=imsizex
	nclasses=dataset.n_classes
	y_ints = [y.argmax() for y in dataset.Y_train]
	class_w = class_weight.compute_class_weight('balanced',
                                                 np.unique(y_ints),
                                                 y_ints)
	
	for x in range(20):
		print y_ints[x]

	# KERAS code
	
	channel_r = Input(shape=(imsizex, imsizey,1),name='red_channel')
	channel_g = Input(shape=(imsizex, imsizey,1),name='green_channel')
	channel_b = Input(shape=(imsizex, imsizey,1),name='blue_channel')
	

	nf=8  #8
	fz=3  # 5
	nf2=6 #16
	channel_1 = Conv2D(nf, (fz, fz), padding='same', activation='relu', name='conv2d_red')(channel_r)
	#channel_1= Conv2D(nf2, (3, 3), padding='same', activation='relu')(channel_1)
	c1 = keras.layers.add([channel_1, channel_r]) # ressonant
	
	channel_2 = Conv2D(nf, (fz, fz), padding='same', activation='relu', name='conv2d_green')(channel_g)
	#channel_2= Conv2D(nf2, (3, 3), padding='same', activation='relu')(channel_2)
	c2 = keras.layers.add([channel_2, channel_g]) # resonant
	
	channel_3 = Conv2D(nf, (fz, fz), padding='same', activation='relu', name='conv2d_blue')(channel_b)
	#channel_3= Conv2D(nf2, (3, 3), padding='same', activation='relu')(channel_3)
	c3 = keras.layers.add([channel_3, channel_b])  #resonant
	
	merge = concatenate([c1, c2, c3])
	
	merge=BatchNormalization()(merge) ####
	
	merge = Conv2D(nf2, (fz, fz), padding='same', activation='relu', name='conv2d_merge')(merge)
	merge=BatchNormalization()(merge)
	
	
	flatmxp=Flatten()(merge)
	
	inter = Dense(64, activation='relu')(flatmxp)
	inter = Dropout(0.5)(inter)
	inter=BatchNormalization()(inter)
	
	inter = Dense(16, activation='relu')(inter) ### 16
	#inter = Dropout(0.5)(inter)
	
	output = Dense(nclasses, activation='softmax', name='out_softmax')(inter)
	
	model = Model(inputs=[channel_r, channel_g, channel_b], outputs=output)

	nadam=Nadam(lr=0.002, beta_1=0.9, beta_2=0.999, epsilon=None, schedule_decay=0.004)
	sgd = SGD(lr=0.01, decay=1e-6, momentum=0.98, nesterov=True)
	#rmsp=RMSprop(lr=0.001, rho=0.9, epsilon=None, decay=0.0)  ######
	
	rmsp=RMSprop(lr=0.01, rho=0.9, epsilon=None, decay=1e-6) 

	#mean_squared_error  categorical_hinge categorical_crossentropy
	model.compile(loss='categorical_crossentropy', optimizer=rmsp, metrics=['accuracy'])
	#model.compile(loss='categorical_crossentropy', optimizer=rmsp, metrics=['accuracy'])


	# summarize layers
	print(model.summary())
	# plot graph
	if fold==0:
		plot_model(model, to_file=save_pred+'.png')
  
	# fit the model =================================================
	model.fit([dataset.X_train[:,:,:,0].reshape(ndata, imsizex, imsizey,1),dataset.X_train[:,:,:,1].reshape(ndata, imsizex, imsizey,1),dataset.X_train[:,:,:,2].reshape(ndata, imsizex, imsizey,1)], dataset.Y_train, batch_size=bsize, epochs=training_steps, verbose=1,class_weight=class_w)

	#/save the model and weights*/
	model.save(save_pred+'_model'+str(fold)+'.h5');
	model.save_weights(save_pred+'_'+str(fold)+'ws.h5');
	
	# evaluate the model
	datatest=[dataset.X_test[:,:,:,0].reshape(ntest, imsizex, imsizey,1),dataset.X_test[:,:,:,1].reshape(ntest, imsizex, imsizey,1),dataset.X_test[:,:,:,2].reshape(ntest, imsizex, imsizey,1)]
	score = model.evaluate(datatest, dataset.Y_test, batch_size=bsize)
	print(score)
  
	prediction = model.predict(datatest)
	pred=np.argmax(prediction, axis=1)
	act=np.argmax(dataset.Y_test, axis=1)
	

	y_actu = pd.Series(act, name='Actual')
	y_pred = pd.Series(pred, name='Predicted')
	df_confusion = pd.crosstab(y_actu, y_pred)
	
	df = pd.DataFrame(act,pred)
	
	print df_confusion
	
	df.to_csv(save_pred+'_'+str(fold)+'pred.txt',columns=None, header=False, index=True, index_label=None)
	

  
  
if __name__ == '__main__':
	main()



